
# q-expansion of newform 276.6.a.b, downloaded from the LMFDB on 29 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 276
weight = 6
poly_data = [7047, -168, -223, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[-2, 4, 0, 0], 1], [[-1470, -268, 10, 1], 33], [[-950, 114, 12, -1], 11]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 0], [9, 0, 0, 0], [-13, -1, -1, 0], [-15, 3, 0, 1], [-98, 0, 7, -7], [-92, -7, 5, 9], [-207, -3, 8, -25], [-73, 13, 29, 12], [-529, 0, 0, 0], [-2500, 34, 16, -2], [-1818, -24, -134, 44], [-4146, 120, -87, 13], [-5094, -116, -78, 82], [-4521, 5, 107, -46], [-7764, -275, -189, 97], [-2875, -185, -191, 16], [-4328, -359, -339, -277], [34, 350, 53, -63], [-3355, -409, -507, -142], [-17764, -64, -666, 58], [1350, 452, 304, 892], [-26333, -975, 208, 387], [-34318, 1758, 775, -447], [-29713, 245, 1308, 205], [-26444, -326, -686, -1724], [-26534, 198, -398, -1942], [-35989, -1743, 386, 413], [-64514, -804, -1871, 995], [-12008, -784, -2819, 1727], [-12845, -1487, 656, -1091], [29150, 388, 410, 2316], [-11780, 3734, 4032, 684], [-71381, 3211, 4752, -4399], [49072, 1110, -3594, 1166], [6375, -3531, -287, 2290], [90120, 5732, 6836, -3940], [74398, 7428, -2543, 1557], [404214, 974, 542, -2632], [77888, -6538, 3944, -1356], [121260, 7202, -4128, 6302], [221628, -3649, 865, 3447], [69484, -114, -4469, 6909], [-125654, -9088, -13988, -7166], [439804, -11111, -981, -1821], [-20572, -2050, 11598, -5972], [431771, -11819, -6624, -1757], [471158, 10552, 308, 10818], [612008, -3430, -7014, -6202], [-56480, -3162, 6735, -8741], [-86704, 6312, -3507, 2655], [-256454, -5502, -3606, 16942], [309988, -13258, 8010, -6174], [170664, 15924, 21956, -13242], [134376, 1468, -12065, 18191], [-213586, -84, -13498, -1622], [-136678, 16664, -5246, -3496], [-189124, 26258, 31286, -11076], [273610, -8328, -19050, 7460], [-724206, -10300, 5810, -17482], [-183521, 30691, 29082, 29999], [373261, -34573, -16249, -204], [-713679, -9415, -1297, -6338], [538226, 33564, 26328, -21678], [-29756, 8793, -5519, -16897], [-831054, 8990, 5156, 34160], [-1029796, 7060, 13654, -1832], [756030, 43182, 48006, 31840], [-1185852, -13566, -51088, 27882], [755008, -43753, 42097, -12965], [-3188, 14085, -35361, -7573], [-578872, 52086, 41452, -13494], [-472898, 55052, -2532, 8910], [597851, 17429, -3512, -6069], [-259506, -2356, 18189, -16263], [132923, -9595, -63055, -15092], [-1199138, -67582, 31716, 4434], [-2811279, 32993, 30633, -31840], [-929154, -46344, -3840, -68816], [-4915, 26789, 9888, -35117], [270708, 8779, 86885, -40175], [1375706, -26246, -36897, 10953], [-1836972, -71834, -32143, 60547], [-650844, -44542, -67164, -18676], [-1655842, 45896, 42734, 50122], [1446040, -21714, 23634, -274], [754928, -50900, -122504, 106156], [93082, 11566, -18188, -60576], [1233588, 78064, 936, 74366], [-1699944, -30836, -99888, 16222], [-2252884, -58116, 128, -96724], [-420908, 52920, -22465, -44541], [697960, 5250, 113162, -37002], [2347312, -125552, -94276, 65684], [2545024, -13591, -47001, -37755], [914966, -33038, -21214, 86580], [133408, -17918, 86172, -115024], [1795800, -104960, 7182, -16052], [1043187, -150363, -22574, 33607], [-273631, -54653, 2841, 35050], [-1069596, 27447, 95725, -20251], [3616390, 61148, -70112, 19622], [7829613, -87675, -12307, 5528], [405656, 46998, 184295, 51355], [6922113, 13219, 48262, -60703], [-1097887, -53653, -199899, 12110], [1645972, 170661, -108137, 26463], [3256276, -175954, 13060, -10592], [-1266994, 191790, -27408, 48536], [-2375852, 228246, 21230, -26418], [-3171020, 271529, 174107, 121443], [-4367180, 79544, 153948, -64856], [-8904706, -48416, -166233, 68899], [2738699, -82695, -3636, -163447], [-2503855, -37897, 26879, 151720], [-3358057, -145291, -199986, 133757], [-572295, -258831, -92984, 108823], [-3322753, -170923, 80123, -132062], [-5676472, 79799, -71525, 75089], [-1226958, 282920, -223356, 53888], [-1008332, -56908, -150009, -107325], [-3839790, -77752, -100337, -165497], [-4644176, -122497, -29911, -47559], [404319, 251729, 292665, -128326], [-3867704, -101074, -27480, -248640], [-8986318, -84664, 49728, -59230], [-1682769, -150973, -29487, -60658], [-6423212, 127696, -21573, 23409], [-464296, 364307, 257907, 170569], [-9659799, -149269, 3772, -280931], [-6386848, 71058, -272503, 141791], [-50936, 316520, 159116, 276024], [-7224354, 302666, 77364, 13706], [1617641, 300643, 413914, -30301], [-9039852, 250594, 252715, -88035], [3759104, 171582, -62700, 67730], [-3491126, 165174, 27032, 66148], [-8636437, 269905, 298771, 18064], [9766647, 310829, 99151, 103534], [5698801, 51515, 181423, -43958], [3706568, 237444, -211926, 180092], [11822986, 244008, -147816, -27094], [10559414, 338026, 304614, -204246], [5836186, -75942, 1732, 262254], [-12492244, -172276, 175179, -83897], [-3510988, -30341, 80557, -57523], [-10104610, 87088, 311202, -205788], [-2932906, -444412, -350092, 411916], [16661766, 64634, 49170, -306406], [-3905704, -197770, -461626, 155398], [-1000128, -627755, -196687, -99693], [4271054, 242628, 322594, -55934], [-7607073, -147673, 139062, 379355], [2802114, 9242, 203842, -268280], [-7100648, -115429, -197487, 138039], [14223985, 169227, 530171, -129788], [-20042736, -374158, -388376, 291252], [1304247, 484257, 421656, -313961], [7886610, -642856, -380334, 256010], [3856814, -39384, 442892, -252952], [281011, 717343, 307611, 294356], [4464604, 616119, -41739, 18763], [-9364979, 612455, 385226, 165089], [-7585534, 142560, -638358, 473640], [-2450412, 96590, -232621, 15331], [21707007, 132529, 10278, -292817], [-14861376, 687458, 613284, 201464], [7874442, 7786, 349400, 78538], [13682428, -704115, -617593, -406085], [1226924, 557590, 979130, -372036], [13345838, 653492, 980608, 288176], [17522392, 223920, 995363, -188517], [14184822, -244552, -277510, -235270], [-8325706, -394790, -43076, -225110], [7510472, -623952, 112670, -320200], [6324506, -557584, -499302, -492664], [-4174563, 712981, 333546, 142669], [24978572, -993594, -501186, -311718], [-4309182, -233894, 217080, -747608], [22681383, 368493, -92550, 359305], [-5431818, -632170, -827349, 585483], [31359230, -371392, 354306, -43972], [-527416, -12322, 46645, 194609], [34944394, 452076, 595370, -576458], [28645828, 704628, -372054, 665392], [-23180384, -544766, -685964, 79968], [32993501, 186803, 37131, -314606], [11657918, -1079020, -703212, 451244], [-8526757, -786239, -694385, 488162], [-23190770, 480016, 308496, -573828], [-26330120, 664815, 206297, 326847], [-33839238, 577160, 38592, 56136], [-7342408, 536834, 677968, 1013840], [29445603, 401337, 1238571, 171910], [-10938138, -440812, -129094, 282898], [2923602, 769538, -12565, 29965], [12679381, -379689, -335592, -374849], [9036338, 958702, -92934, 434758], [-56606452, -180686, 215085, -374009], [-22947023, -1082391, -288626, -119539], [-33755884, -5892, 803228, 172504], [-19912263, -642463, -496275, 676436], [6614706, 846242, -525332, 569302], [25755530, -9812, 1464582, -448142], [-24334620, -310302, -174720, -1207430], [-61720326, -332036, 489015, -696443], [6804620, -712828, 190058, 308028], [-31552847, 163091, -1448148, 310285], [47227528, 387041, 642221, 319955], [-33284502, -1168590, 319428, 258048], [4379874, 745826, 1699134, 84924], [-17584006, -1040364, -556544, 1236448], [-33826834, -1566742, 809000, -1226340], [26856081, -1316285, -732154, 406327], [12563398, 799870, -178247, -448401], [-12109188, 281156, 251870, -14390], [-65199782, -406382, -155972, -207080], [8526892, -782384, -282592, -1335820], [-33903864, 2336096, 1543230, -504400], [7336336, -1026448, 563172, 513996], [31189286, -641410, 39288, 212692], [-15736828, -946927, -2161007, -661683], [-56929183, -301377, 1164110, -1021945], [49350284, -1817088, -719346, -615508], [28558087, 855225, 2337526, -173067], [56223932, 1426858, -6930, -904662], [-97051192, -611505, 396691, -747977], [28267099, -507503, -1577744, -452547], [-78741168, 1433705, 935283, 191333], [19469361, 982131, 1242090, -385901], [27750968, 405732, -895676, 1232832], [-38219980, -70525, -3483, 216389], [-10178131, -237613, 1793967, -1470460], [-72338467, -1078189, -1549678, 426055], [32386384, 848728, 1215616, 203802], [20677889, -1424465, -1191577, -1619952], [84729216, 246710, -704112, 1113572], [-110811216, -582678, -239818, 914508], [-13345115, -309501, 99935, 2056014], [13631996, -2185516, -2389198, 1121730], [44104896, 851513, 1791799, -895119], [103735132, -1241202, -517009, -248625], [-16276046, 2023798, 2397870, -929244], [-60284454, 485122, 177800, -283694], [-6344376, 232635, -1083433, -413397], [126907314, 24466, -441234, -96782], [3475020, 1454008, 1540812, -1005176], [-37086702, 497002, -2181996, 40334], [101865362, -726630, 814331, -553043], [51028413, -997445, -1092397, -852656], [-101915452, 17900, 236256, 1122580], [-53531760, 874268, 744721, -11117], [-20962475, -1452691, -186226, -1881343], [88819722, 485338, 1233972, -990818], [-63313592, -845932, 264758, -992664], [-100710602, -252650, -1397508, -425560], [-6171632, 1870536, 77168, 16644], [-56732008, 338710, 760683, -564287], [-86911879, -539209, -2919163, 354766], [-7172908, -1731860, -892460, -1008578], [9652292, -2832303, -3001869, 1191243], [21804423, 2336761, -676182, 1433753], [34410022, 579596, -819339, 1053783], [-44570710, -1075196, -1335150, 1248142], [-36823018, -332760, 191355, 682311], [112396362, 15050, 475806, 152954], [-39960815, -761377, 389875, 446900], [-54470511, 596939, -993053, 1074846], [39912441, 770745, 1798906, 1915465], [-1617699, 2632739, -78901, -468568], [46583856, 1235940, 3118708, 842770], [84235350, -192588, 109274, -642510], [-75743417, 1272761, 2487843, -2172566], [26033416, -1978910, 1816576, -420770], [-76545223, 1827203, -1124342, 809307], [143001166, -415712, -1375772, -855176], [68942800, -3972106, -736154, -919606], [182424996, 92809, -968631, 843595], [13868638, 2177988, 1362516, -550132], [-25337034, -2635976, -742796, -1630008], [-87758398, -892252, 2151759, -2104355], [26430332, 841557, -30555, 3675707], [-10801201, -2229331, 871602, -366751], [68456850, -622392, -1770432, 1294710], [40320934, -1250068, -1110265, -300381], [-6187986, -367644, -724364, -324646], [-108795692, 954855, 1513695, -1742591], [-79266104, -2925868, -4024104, 1749994], [-15831235, 2576781, 2039749, 3235416], [-6961976, -840064, -3168336, 3250576], [-127668928, 2500526, 772110, -284276], [-23445869, -2543739, -1930583, 2715014], [-85411030, -1185232, 476217, -6305], [-13356640, -1243802, 91890, -1232616], [-110599528, -542290, -1343949, 2697919], [-23457284, 2241549, 2567295, 202331], [-86755077, -231245, -2343933, -132260], [-156164423, -376569, 2036456, -3079583], [10557746, -4346786, -2517666, -1249890], [-176115652, -1934512, -2897580, -1218620], [44115484, 2975148, 2693260, 665484], [116016542, 1342198, -1170428, -1754140], [-162822579, 511209, -483531, -333000], [-197015367, 781795, 2105588, 685981], [-81795484, -1863105, 1546951, -2317347], [-24818965, -1104815, -3178371, 1802536], [168430932, 352121, -209977, 2423159], [-89815056, 1463622, 1213506, 512250], [39761584, 710760, 2640021, -669841], [11724558, 3739380, -1013994, -622368], [129578832, 311735, -3321745, 3756635], [14600056, -1689087, -3279955, -1214241], [-163637931, -2565123, 714599, 768554], [-54768541, -1926563, 741558, 3731091], [-97036560, -1539792, -3450372, 321760], [-177431488, 4894824, 2705722, 1531290], [-116552500, -2801328, -2995146, 1512876], [76191688, -2881486, 4507910, -1213594], [25309972, -21045, -1834231, 344859], [94016848, 1976544, 2529626, 3104884], [-88801774, 1650124, -6125146, 712070], [86748875, -3413987, 1189391, -2185062], [35796482, -229952, -3960478, -2533902], [-236580642, 1252908, 1687118, -513202], [-2101536, 4814348, 2929904, -1613272], [-85333075, 2793499, -515358, 777873], [79709688, 2707598, -2973338, 4472284], [-110711525, -502075, 2185187, -3654546], [126375900, 693578, -2933398, 3720254], [-126911188, -2089350, -7150894, 2385994], [31130481, 3535207, -577385, -2792878], [142806786, 1506532, 3543438, -308182], [270846150, -1595894, -3081774, 241582], [-100678236, -1229728, 2742216, -1428444], [2201208, -1155276, -4778394, 3330162], [-5858359, 3234011, 1008507, -38692], [-54068664, -972109, -2863811, 1460823], [128217834, 1140892, 694329, -1755871], [67637975, 273043, 856618, -1272977], [102565052, 1040313, 402123, 981483], [144773631, -4701631, -1383378, 2150685], [293131192, 588059, -559981, -2694321], [5205649, -486383, 1376600, 2982343], [63037222, -4315906, -5366384, -942044], [-65592267, -4907093, -5477308, 760081], [-89255445, -4892759, 775883, -2738592], [-129520760, -912552, 1638412, 1790408], [104218156, -8484667, -3725907, -1614167], [88093762, 4349612, 3563116, -2975122], [352599856, 1938794, -1337820, 632352], [-23306739, 1874785, -4385093, 3070718], [72605288, 3452304, -1981832, -1202012], [199379564, -907489, -3566011, -1274447], [110871300, -642092, -1804884, -1973426], [-125986513, 5623837, 3338924, -1058181], [234414382, -7751136, -4641441, 98603], [-146642316, -4136276, -2783166, 2483848], [97059922, 2097314, 163364, 1090478], [-95940588, 4504546, -2063127, 1709209], [20455316, 3024194, -987112, -3365214], [-199652200, 197893, 4017083, -2248795], [268936150, 1725430, 3870473, -872935], [-12948596, -1686, -896782, 5063556], [4824768, -659322, 4604308, -3715208], [-151626732, -2459104, 4915005, -6189871], [-1765548, 6251822, 2019182, -2411062], [65340242, 1166284, -2341424, -1261024], [231691130, -1056914, -2156540, -2930996], [-226299269, 6328585, -1879304, 812779], [-246585666, 1333266, -818328, -1917776], [-66491944, 8937629, 5661131, -887727], [-159670526, -7135002, -3369230, -1456632], [-6574132, -8848542, -1206272, -2610384], [-296264141, -1710501, -1488939, -274970], [-268541373, -35783, -3231754, -1796447], [267804142, 3025000, 1174240, 5304784], [17905640, 8121778, 7227608, -3520816], [-107765788, 9450348, 2693982, 3327002], [-218569638, 372986, 1281858, 5933742], [62084611, -2722507, 5056554, 1932327], [69540716, -7534899, -8756645, -3187277], [-109134695, 2061755, -3724797, 5732934], [-54434881, -709353, -4137888, -4700971], [199758170, -2309380, 2283698, 3341008], [85345326, -828246, -1502736, 5273520], [-127752293, 3955897, 2042865, 1285452], [-145102792, 9082978, -970358, 2589646], [-170684850, -5946966, -8242696, 1778934], [59494474, -3909292, -3628206, -6996866], [2291997, 850043, 6640413, -2965878], [23510836, 180536, 1708678, -1697366], [-377963992, -1224464, -3080826, -1834472], [-57985240, 395926, -8929766, 50356], [-233506000, -5782213, -205509, 3449557], [-12183286, 9001608, 42368, -2150614], [365787772, 701192, -3866838, 1225758], [-167625844, -5076732, -4391908, -4120894], [111822389, 2875267, 1015110, -4628573], [-167077029, -830083, -9998766, -137089], [-305842361, -2334571, 1853979, 722890], [49587634, 7468850, -229000, 6857576], [312687886, 4962492, -2178376, -2975360], [-130676868, -7228614, -1785310, 2158748], [37304459, 8710653, 8079412, 7313047], [321512545, 1593179, 4746538, -3496427], [-1626802, 6567536, 3725930, 2489466], [-99865808, 7703472, -3618068, -366540], [-193235160, -8432890, -737997, 1815921], [-257202592, 1406476, 3150176, -5498422], [-404949295, 1507295, -3186477, 2006636], [-106086460, -3799663, 54195, 5978629], [24161336, 2712627, 3408041, 2159033], [-106714822, 4771702, -4451640, 1142664], [-424098842, -2011018, -6165249, 475053], [-175569043, -1726061, -311391, 1232062], [-344391408, -7921652, 3123826, 6995004], [39557844, 9755886, 7598220, 3865098], [83917604, -30923, -983757, -7997799], [-109761966, 2387858, 2018160, -3066942], [52656123, 1616985, 6985902, -7378917], [451088078, 1785198, 8103886, 179172], [-39538935, 8994279, -8320021, 947702], [232596542, -4550986, 2521555, -3456957], [4930612, -7747668, -11588712, -9817652], [-93649824, -2605505, -5611725, 5958757], [-96122622, -7822228, 2497328, 6180608], [-149288270, 4514324, 11181232, -2710988], [108080692, 5741424, 3168177, 6633181], [-431166822, -1848116, 1477300, -2528548], [32506222, 7285634, 14300074, -10903636], [149147124, -2301873, 2422513, -4443821]]
